import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as ss
import pystan
import seaborn as sb

flnm = input('Data File Name (*.xlsx) = ')
xlsx = pd.ExcelFile(flnm)                   #   入力ファイル名の設定
data = pd.read_excel(xlsx)                  #   読み込み

N_no = data.values[0][1]
N_yes = data.values[0][2]
S_no = data.values[1][1]
S_yes = data.values[1][2]
print('Noise  no:', N_no, '  yes:', N_yes)
print('Signal no:', S_no, '  yes:', S_yes)

Data = {'N_no': N_no, 'N_yes': N_yes, 'S_no': S_no, 'S_yes': S_yes}

sm = pystan.StanModel(file = 'SDT2Cat.stan')
fit = sm.sampling(data = Data, n_jobs = 1)

print(fit)

C_mean = fit['C'].mean()
C_p025, C_Q1, C_med, C_Q3, C_p975 = np.percentile(fit['C'], [2.5, 25, 50, 75, 97.5])
d_mean = fit['d'].mean()
d_p025, d_Q1, d_med, d_Q3, d_p975 = np.percentile(fit['d'], [2.5, 25, 50, 75, 97.5])

sb.kdeplot(fit['C'], label = 'C')
sb.kdeplot(fit['d'], label = "d'")
plt.yticks([])
plt.title("C(med) = {0:.3}    d'(med) = {1:.3}".format(C_med, d_med), fontsize = 16)
plt.legend(fontsize = 14)
plt.show()

Cs = np.arange(-5, 5, 0.01)
FAs = [1.0 - ss.norm.cdf(v, 0.0, 1.0) for v in Cs]
Hits = [1.0 - ss.norm.cdf(v, d_med, 1.0) for v in Cs]
Hitsp025 = [1.0 - ss.norm.cdf(v, d_p025, 1.0) for v in Cs]
HitsQ1 = [1.0 - ss.norm.cdf(v, d_Q1, 1.0) for v in Cs]
HitsQ3 = [1.0 - ss.norm.cdf(v, d_Q3, 1.0) for v in Cs]
Hitsp975 = [1.0 - ss.norm.cdf(v, d_p975, 1.0) for v in Cs]
plt.figure(figsize = (5, 5))
plt.plot([0, 1, 1, 0, 0], [0, 0, 1, 1, 0], c = 'k')
plt.plot([0, 1], [0, 1], c = 'k')
plt.fill_between(FAs, Hitsp025, Hitsp975, color = 'y', alpha = 0.3)
plt.fill_between(FAs, HitsQ1, HitsQ3, color = 'g', alpha = 0.3)
plt.plot(FAs, Hits, c = 'b')
plt.plot(N_yes/(N_yes + N_no), S_yes/(S_yes + S_no), 'ro', label = 'Data')
plt.title(("ROC curve: d'(med) = {0:.3}".format(d_med)) +
          '\n95%CI(yellow)   50%CI(green)', fontsize = 16)
plt.xlabel('FA', fontsize = 14)
plt.ylabel('Hit', fontsize = 14)
plt.legend(fontsize = 14)
plt.show()

print('\nC(mean) = {0:.3}'.format(C_mean))
print('    95%CI = [{0:.3}, {1:.3}]'.format(C_p025, C_p975))
print('    median = {0:.3}'.format(C_med))
print('    Q1 = {0:.3},    Q3 = {1:.3}'.format(C_Q1, C_Q3))
print("\nd'(mean) = {0:.3}".format(d_mean))
print('     95%CI = [{0:.3}, {1:.3}]'.format(d_p025, d_p975))
print('     median = {0:.3}'.format(d_med))
print('     Q1 = {0:.3},    Q3 = {1:.3}'.format(d_Q1, d_Q3))
print()
