import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as ss
import scipy.optimize as so
import pystan
import seaborn as sb


flnm = input('Data File Name (*.xlsx) = ')
xlsx = pd.ExcelFile(flnm)                   #   入力ファイル名の設定
data = pd.read_excel(xlsx)                  #   読み込み

X = data.values
K = len(X[0]) - 1
print('K = ', K)
Res_noise = [v for v in X[0][1:]]
Res_signal = [v for v in X[1][1:]]
print('Res_noise =\n', Res_noise)
print('Res_signal =\n', Res_signal)

print('a = \n', np.ones(K))

Data = {'K': K, 'Noise': Res_noise, 'Signal': Res_signal, 'a': np.ones(K)}

sm = pystan.StanModel(file = 'SDTKCat.stan')
fit = sm.sampling(data = Data, n_jobs = 1)

print(fit)

sb.kdeplot(fit['d'], color = 'b', label = '$\mu_s$')
for k in range(K-1):
    sb.kdeplot(fit['C'].T[k], linestyle = '--', label = 'C{}'.format(k+1))
plt.yticks([])
plt.title('Posterior distributions\n' +
          '$\mu_s$ and Cs', fontsize = 16)
plt.legend(fontsize = 12)
plt.show()

sb.kdeplot(fit['sgm'])
plt.title('Posterior distribution of $\sigma_s$', fontsize = 16)
plt.show()

d_mean = fit['d'].mean()

d_p025, d_Q1, d_med, d_Q3, d_p975 = np.percentile(fit['d'], [2.5, 25, 50, 75, 97.5])

C_means = []
for k in range(K - 1):
    C_means.append(fit['C'][:, k].mean())
C_pest = []
for k in range(K - 1):
    C_pest.append(np.percentile(fit['C'][:, k], [2.5, 25, 50, 75, 97.5]))

sgm_mean = fit['sgm'].mean()
sgm_p025, sgm_Q1, sgm_med, sgm_Q3, sgm_p975 = np.percentile(fit['sgm'], [2.5, 25, 50, 75, 97.5])

n = 100

Param_sgm = fit['sgm']
Param_d = fit['d']

smpl_pos = np.random.choice(np.arange(len(Param_sgm)), size = n, replace = False)

plt.figure(figsize = (5, 5))
plt.plot([0, 1, 1, 0, 0], [0, 0, 1, 1, 0], c = 'k')
plt.plot([0, 1], [0, 1], c = 'k')

Cs = np.arange(-5, 5, 0.01)
for i in range(n):
    if i % 10 == 0:
        if i == 0:
            print('step-{0}/{1}'.format(i, n), end = '')
        else:
            print('\rstep-{0}/{1}'.format(i, n), end = '')
    FAs = [1.0 - ss.norm.cdf(v, 0.0, 1.0) for v in Cs]
    Hits = [1.0 - ss.norm.cdf(v, Param_d[smpl_pos[i]], Param_sgm[smpl_pos[i]]) for v in Cs]
    plt.plot(FAs, Hits, color = 'y', alpha = 0.2)
print()
FAs = [1.0 - ss.norm.cdf(v, 0.0, 1.0) for v in Cs]
Hits = [1.0 - ss.norm.cdf(v, d_med, sgm_med) for v in Cs]
plt.plot(FAs, Hits, color = 'b', linewidth = 3, solid_capstyle = 'round')
CFA = [1.0 - ss.norm.cdf(C_pest[k][2], 0.0, 1.0) for k in range(K-1)]
CHit = [1.0 - ss.norm.cdf(C_pest[k][2], d_med, sgm_med) for k in range(K-1)]
plt.plot(CFA, CHit, 'gs', label = 'Model')

Res_noise = np.array(Res_noise)
Res_signal = np.array(Res_signal)
sum_noise = Res_noise.sum()
sum_signal = Res_signal.sum()
cum_noise = 0
cum_signal = 0
for k in range(K - 1):
    cum_noise += Res_noise[K-1-k]
    cum_signal += Res_signal[K-1-k]
    if k == 0:
        plt.plot(cum_noise/sum_noise, cum_signal/sum_signal, 'ro', label = 'Data')
    else:
        plt.plot(cum_noise/sum_noise, cum_signal/sum_signal, 'ro')
plt.title('ROC curves\n' +
          '$\mu_s$(med) = {0:.3}   $\sigma_s$(med) = {1:.3}'.format(d_med, sgm_med),
          fontsize = 16)
plt.legend(fontsize = 14)
plt.show()

print("\nd'")
print('      mean = {0:.3}'.format(d_mean))
print('      95%CI = [{0:.3}, {1:.3}]'.format(d_p025, d_p975))
print('      Q1 = {0:.3},   Med. = {1:.3},   Q3 = {2:.3}'.format(d_Q1, d_med, d_Q3))

print('\nsgm')
print('      mean = {0:.3}'.format(sgm_mean))
print('      95%CI = [{0:.3}, {1:.3}]'.format(sgm_p025, sgm_p975))
print('      Q1 = {0:.3},   Med. = {1:.3},   Q3 = {2:.3}'.format(sgm_Q1, sgm_med, sgm_Q3))

for k in range(K-1):
    print()
    print('C{}'.format(k+1))
    print('      mean = {0:.3}'.format(C_means[k]))
    print('      95%CI = [{0:.3}, {1:.3}]'.format(C_pest[k][0], C_pest[k][4]))
    print('      Q1 = {0:.3},   Med. = {1:.3},   Q3 = {2:.3}'.format(
                    C_pest[k][1], C_pest[k][2], C_pest[k][3]))

print()

