data {
    int K;
    int<lower = 0> Noise[K];
    int<lower = 0> Signal[K];
    vector[K] a;
}
parameters {
    simplex[K] theta;
    real d;
}
transformed parameters {
    simplex[K] p_noise;
    simplex[K] p_signal;
    ordered[K-1] C;
    real v;
    v = theta[1];
    for (i in 1:(K-1)) {
        C[i] = logit(v);
        v += theta[i+1];
    }
    p_noise[1] = normal_cdf(C[1], 0.0, 1.0);
    p_signal[1] = normal_cdf(C[1], d, 1.0);
    for (k in 2:(K-1)) {
        p_noise[k] = normal_cdf(C[k], 0.0, 1.0) - normal_cdf(C[k-1], 0.0, 1.0);
        p_signal[k] = normal_cdf(C[k], d, 1.0) - normal_cdf(C[k-1], d, 1.0);
    }
    p_noise[K] = 1.0 - normal_cdf(C[K-1], 0.0, 1.0);
    p_signal[K] = 1.0 - normal_cdf(C[K-1], d, 1.0);
}
model {
    d ~ normal(0.0, 1000.0);
    theta ~ dirichlet(a);
    Noise ~ multinomial(p_noise);
    Signal ~ multinomial(p_signal);
}
